@@ -1,5 +1,6 @@
 import contextlib
 import copy
+import json
 import random
 import time
 import unittest
@@ -7,22 +8,17 @@ import zlib
 
 import numpy
 
+from nion.instrumentation import camera_base
+from nion.instrumentation.test import AcquisitionTestContext
+from nion.instrumentation.test import HardwareSource_test
 from nion.swift import Application
-from nion.swift import DocumentController
 from nion.swift import Facade
-from nion.swift.model import DocumentModel
-from nion.swift.model import HardwareSource
+from nion.swift.model import Graphics
 from nion.swift.model import Metadata
-from nion.swift.test import HardwareSource_test
-from nion.swift.test import TestContext
 from nion.ui import TestUI
-from nion.utils import Event
 from nion.utils import Geometry
 from nion.utils import Registry
-from nion.instrumentation import camera_base
 from nionswift_plugin.nion_instrumentation_ui import CameraControlPanel
-from nionswift_plugin.usim import InstrumentDevice
-from nionswift_plugin.usim import CameraDevice
 
 """
 # running in Swift
@@ -41,14 +37,6 @@ class TestCameraControlClass(unittest.TestCase):
         self.app = Application.Application(TestUI.UserInterface(), set_global=False)
         self.source_image = numpy.random.randn(1024, 1024).astype(numpy.float32)
         self.exposure = 1.0
-        HardwareSource.HardwareSourceManager().hardware_sources = []
-        HardwareSource.HardwareSourceManager().hardware_source_added_event = Event.Event()
-        HardwareSource.HardwareSourceManager().hardware_source_removed_event = Event.Event()
-        # HardwareSource.HardwareSourceManager()._reset()
-
-    def tearDown(self):
-        HardwareSource.HardwareSourceManager()._close_hardware_sources()
-        HardwareSource.HardwareSourceManager()._close_instruments()
 
     def _acquire_one(self, document_controller, hardware_source):
         hardware_source.start_playing()
@@ -65,51 +53,21 @@ class TestCameraControlClass(unittest.TestCase):
             self.assertTrue(time.time() - start_time < TIMEOUT)
         document_controller.periodic()
 
-    def __test_context(self, initialize: bool=True, is_eels: bool=False):
+    def __test_context(self, is_eels: bool=False):
+        return AcquisitionTestContext.test_context(is_eels=is_eels, camera_exposure=0.04)
 
-        class SimpleTestContext(TestContext.MemoryProfileContext):
-            def __init__(self, camera_hardware_source, initialize):
-                super().__init__()
-                self.document_controller = self.create_document_controller(auto_close=False)
-                self.document_model = self.document_controller.document_model
-                self.hardware_source = camera_hardware_source
-                state_controller = CameraControlPanel.CameraControlStateController(camera_hardware_source, self.document_controller.queue_task, self.document_model)
-                if initialize:
-                    state_controller.initialize_state()
-                self.state_controller = state_controller
+    def __create_state_controller(self, acquisition_test_context: AcquisitionTestContext.AcquisitionTestContext, *,
+                                  initialize: bool = True) -> CameraControlPanel.CameraControlStateController:
+        state_controller = CameraControlPanel.CameraControlStateController(acquisition_test_context.camera_hardware_source,
+                                                                           acquisition_test_context.document_controller.queue_task,
+                                                                           acquisition_test_context.document_model)
+        if initialize:
+            state_controller.initialize_state()
+        acquisition_test_context.push(state_controller)
+        return state_controller
 
-            def close(self):
-                self.document_controller.close()
-                self.state_controller.close()
-                super().close()
-
-        # this is simulator specific. replace this code but be sure to set up self.exposure and blanked and positioned
-        # initial settings.
-        self.exposure = 0.04
-
-        instrument = InstrumentDevice.Instrument("usim_stem_controller")
-        Registry.register_component(instrument, {"stem_controller"})
-
-        camera_id = "usim_ronchigram_camera" if not is_eels else "usim_eels_camera"
-        camera_type = "ronchigram" if not is_eels else "eels"
-        camera_name = "uSim Camera"
-        camera_settings = CameraDevice.CameraSettings(camera_id)
-        camera_device = CameraDevice.Camera(camera_id, camera_type, camera_name, instrument)
-        camera_hardware_source = camera_base.CameraHardwareSource("usim_stem_controller", camera_device, camera_settings, None, None)
-        if is_eels:
-            camera_hardware_source.features["is_eels_camera"] = True
-            camera_hardware_source.add_channel_processor(0, HardwareSource.SumProcessor(((0.25, 0.0), (0.5, 1.0))))
-        camera_hardware_source.set_frame_parameters(0, camera_base.CameraFrameParameters({"exposure_ms": self.exposure * 1000, "binning": 2}))
-        camera_hardware_source.set_frame_parameters(1, camera_base.CameraFrameParameters({"exposure_ms": self.exposure * 1000, "binning": 2}))
-        camera_hardware_source.set_frame_parameters(2, camera_base.CameraFrameParameters({"exposure_ms": self.exposure * 1000 * 2, "binning": 1}))
-        camera_hardware_source.set_selected_profile_index(0)
-
-        HardwareSource.HardwareSourceManager().register_hardware_source(camera_hardware_source)
-
-        return SimpleTestContext(camera_hardware_source, initialize)
-
-    def _test_context(self, initialize: bool=True, is_eels: bool=False):
-        return self.__test_context(initialize=initialize, is_eels=is_eels)
+    def _test_context(self, *, is_eels: bool = False) -> AcquisitionTestContext.AcquisitionTestContext:
+        return self.__test_context(is_eels=is_eels)
 
     ## STANDARD ACQUISITION TESTS ##
 
@@ -119,47 +77,47 @@ class TestCameraControlClass(unittest.TestCase):
     def test_acquiring_frames_with_generator_produces_correct_frame_numbers(self):
         with self.__test_context() as test_context:
             document_controller = test_context.document_controller
-            hardware_source = test_context.hardware_source
+            hardware_source = test_context.camera_hardware_source
             HardwareSource_test._test_acquiring_frames_with_generator_produces_correct_frame_numbers(self, hardware_source, document_controller)
 
     def test_acquire_multiple_frames_reuses_same_data_item(self):
         with self.__test_context() as test_context:
             document_controller = test_context.document_controller
-            hardware_source = test_context.hardware_source
+            hardware_source = test_context.camera_hardware_source
             HardwareSource_test._test_acquire_multiple_frames_reuses_same_data_item(self, hardware_source, document_controller)
 
     def test_simple_hardware_start_and_stop_actually_stops_acquisition(self):
         with self.__test_context() as test_context:
             document_controller = test_context.document_controller
-            hardware_source = test_context.hardware_source
+            hardware_source = test_context.camera_hardware_source
             HardwareSource_test._test_simple_hardware_start_and_stop_actually_stops_acquisition(self, hardware_source, document_controller)
 
     def test_simple_hardware_start_and_abort_works_as_expected(self):
         with self.__test_context() as test_context:
             document_controller = test_context.document_controller
-            hardware_source = test_context.hardware_source
+            hardware_source = test_context.camera_hardware_source
             HardwareSource_test._test_simple_hardware_start_and_abort_works_as_expected(self, hardware_source, document_controller)
 
     def test_view_reuses_single_data_item(self):
         with self.__test_context() as test_context:
             document_controller = test_context.document_controller
-            hardware_source = test_context.hardware_source
+            hardware_source = test_context.camera_hardware_source
             HardwareSource_test._test_view_reuses_single_data_item(self, hardware_source, document_controller)
 
     def test_get_next_data_elements_to_finish_returns_full_frames(self):
         with self.__test_context() as test_context:
             document_controller = test_context.document_controller
-            hardware_source = test_context.hardware_source
+            hardware_source = test_context.camera_hardware_source
             HardwareSource_test._test_get_next_data_elements_to_finish_returns_full_frames(self, hardware_source, document_controller)
 
     def test_exception_during_view_halts_playback(self):
         with self.__test_context() as test_context:
-            hardware_source = test_context.hardware_source
+            hardware_source = test_context.camera_hardware_source
             HardwareSource_test._test_exception_during_view_halts_playback(self, hardware_source, self.exposure)
 
     def test_able_to_restart_view_after_exception(self):
         with self.__test_context() as test_context:
-            hardware_source = test_context.hardware_source
+            hardware_source = test_context.camera_hardware_source
             HardwareSource_test._test_able_to_restart_view_after_exception(self, hardware_source, self.exposure)
 
     # End of standard acquisition tests.
@@ -168,7 +126,7 @@ class TestCameraControlClass(unittest.TestCase):
         with self.__test_context() as test_context:
             document_controller = test_context.document_controller
             document_model = test_context.document_model
-            hardware_source = test_context.hardware_source
+            hardware_source = test_context.camera_hardware_source
             self._acquire_one(document_controller, hardware_source)
             self._acquire_one(document_controller, hardware_source)
             self._acquire_one(document_controller, hardware_source)
@@ -178,7 +136,7 @@ class TestCameraControlClass(unittest.TestCase):
         with self.__test_context() as test_context:
             document_controller = test_context.document_controller
             document_model = test_context.document_model
-            hardware_source = test_context.hardware_source
+            hardware_source = test_context.camera_hardware_source
             frame_parameters_0 = hardware_source.get_frame_parameters(0)
             frame_parameters_0.binning = 2
             hardware_source.set_frame_parameters(0, frame_parameters_0)
@@ -194,7 +152,7 @@ class TestCameraControlClass(unittest.TestCase):
 
     def test_change_to_profile_with_different_size_during_acquisition_should_produce_different_sized_data(self):
         with self.__test_context() as test_context:
-            hardware_source = test_context.hardware_source
+            hardware_source = test_context.camera_hardware_source
             frame_parameters_0 = hardware_source.get_frame_parameters(0)
             frame_parameters_0.binning = 2
             hardware_source.set_frame_parameters(0, frame_parameters_0)
@@ -205,8 +163,9 @@ class TestCameraControlClass(unittest.TestCase):
             hardware_source.start_playing()
             try:
                 self.assertEqual(hardware_source.get_next_xdatas_to_start()[0].data.shape, hardware_source.get_expected_dimensions(2))
-                time.sleep(self.exposure * 0.1)
+                time.sleep(self.exposure * 1.1)
                 hardware_source.set_selected_profile_index(1)
+                hardware_source.get_next_xdatas_to_start()  # skip one frame during testing to avoid timing variance
                 self.assertEqual(hardware_source.get_next_xdatas_to_start()[0].data.shape, hardware_source.get_expected_dimensions(1))
             finally:
                 hardware_source.abort_playing()
@@ -214,7 +173,7 @@ class TestCameraControlClass(unittest.TestCase):
     def test_changing_frame_parameters_during_view_does_not_affect_current_acquisition(self):
         # NOTE: this currently fails on Orca camera because changing binning will immediately stop acquisition and restart.
         with self.__test_context() as test_context:
-            hardware_source = test_context.hardware_source
+            hardware_source = test_context.camera_hardware_source
             # verify that the frame parameters are applied to the _next_ frame.
             profile_index = 0  # 1, 2 both have special cases that make this test fail.
             frame_parameters = hardware_source.get_frame_parameters(profile_index)
@@ -252,8 +211,8 @@ class TestCameraControlClass(unittest.TestCase):
         with self.__test_context() as test_context:
             document_controller = test_context.document_controller
             document_model = test_context.document_model
-            hardware_source = test_context.hardware_source
-            state_controller = test_context.state_controller
+            hardware_source = test_context.camera_hardware_source
+            state_controller = self.__create_state_controller(test_context)
             hardware_source.start_playing()
             try:
                 hardware_source.get_next_xdatas_to_finish(5)
@@ -270,8 +229,8 @@ class TestCameraControlClass(unittest.TestCase):
         with self.__test_context(is_eels=True) as test_context:
             document_controller = test_context.document_controller
             document_model = test_context.document_model
-            hardware_source = test_context.hardware_source
-            state_controller = test_context.state_controller
+            hardware_source = test_context.camera_hardware_source
+            state_controller = self.__create_state_controller(test_context)
             hardware_source.start_playing()
             try:
                 hardware_source.get_next_xdatas_to_finish(5)
@@ -290,8 +249,8 @@ class TestCameraControlClass(unittest.TestCase):
         with self.__test_context(is_eels=True) as test_context:
             document_controller = test_context.document_controller
             document_model = test_context.document_model
-            hardware_source = test_context.hardware_source
-            state_controller = test_context.state_controller
+            hardware_source = test_context.camera_hardware_source
+            state_controller = self.__create_state_controller(test_context)
             hardware_source.start_playing()
             try:
                 hardware_source.get_next_xdatas_to_finish(5)
@@ -310,7 +269,7 @@ class TestCameraControlClass(unittest.TestCase):
         with self.__test_context() as test_context:
             document_controller = test_context.document_controller
             document_model = test_context.document_model
-            hardware_source = test_context.hardware_source
+            hardware_source = test_context.camera_hardware_source
             frame_parameters_0 = hardware_source.get_frame_parameters(0)
             frame_parameters_0.binning = 4
             hardware_source.set_current_frame_parameters(frame_parameters_0)
@@ -330,8 +289,8 @@ class TestCameraControlClass(unittest.TestCase):
     def test_changing_profile_updates_frame_parameters_in_ui(self):
         with self.__test_context() as test_context:
             document_controller = test_context.document_controller
-            hardware_source = test_context.hardware_source
-            state_controller = test_context.state_controller
+            hardware_source = test_context.camera_hardware_source
+            state_controller = self.__create_state_controller(test_context)
             frame_parameters_ref = [None]
             def frame_parameters_changed(frame_parameters):
                 frame_parameters_ref[0] = frame_parameters
@@ -342,8 +301,8 @@ class TestCameraControlClass(unittest.TestCase):
 
     def test_changing_current_profiles_frame_parameters_updates_frame_parameters_in_ui(self):
         with self.__test_context() as test_context:
-            hardware_source = test_context.hardware_source
-            state_controller = test_context.state_controller
+            hardware_source = test_context.camera_hardware_source
+            state_controller = self.__create_state_controller(test_context)
             frame_parameters_ref = [None]
             def frame_parameters_changed(frame_parameters):
                 frame_parameters_ref[0] = frame_parameters
@@ -356,7 +315,7 @@ class TestCameraControlClass(unittest.TestCase):
 
     def test_binning_values_are_not_empty(self):
         with self.__test_context() as test_context:
-            hardware_source = test_context.hardware_source
+            hardware_source = test_context.camera_hardware_source
             binning_values = hardware_source.binning_values
             self.assertTrue(len(binning_values) > 0)
             self.assertTrue(all(map(lambda x: isinstance(x, int), binning_values)))
@@ -365,8 +324,8 @@ class TestCameraControlClass(unittest.TestCase):
         with self.__test_context() as test_context:
             document_controller = test_context.document_controller
             document_model = test_context.document_model
-            hardware_source = test_context.hardware_source
-            state_controller = test_context.state_controller
+            hardware_source = test_context.camera_hardware_source
+            state_controller = self.__create_state_controller(test_context)
             self._acquire_one(document_controller, hardware_source)
             self.assertEqual(document_model.data_items[0].data_shape, hardware_source.get_expected_dimensions(2))
             state_controller.handle_binning_changed("4")
@@ -377,8 +336,8 @@ class TestCameraControlClass(unittest.TestCase):
         with self.__test_context() as test_context:
             document_controller = test_context.document_controller
             document_model = test_context.document_model
-            hardware_source = test_context.hardware_source
-            state_controller = test_context.state_controller
+            hardware_source = test_context.camera_hardware_source
+            state_controller = self.__create_state_controller(test_context)
             state_controller.handle_change_profile("Snap")
             state_controller.handle_binning_changed("4")
             hardware_source.start_playing()
@@ -397,8 +356,8 @@ class TestCameraControlClass(unittest.TestCase):
         with self.__test_context() as test_context:
             document_controller = test_context.document_controller
             document_model = test_context.document_model
-            hardware_source = test_context.hardware_source
-            state_controller = test_context.state_controller
+            hardware_source = test_context.camera_hardware_source
+            state_controller = self.__create_state_controller(test_context)
             long_exposure = 0.5
             state_controller.handle_change_profile("Snap")
             state_controller.handle_binning_changed("4")
@@ -416,8 +375,8 @@ class TestCameraControlClass(unittest.TestCase):
 
     def test_view_followed_by_frame_uses_correct_exposure(self):
         with self.__test_context() as test_context:
-            hardware_source = test_context.hardware_source
-            state_controller = test_context.state_controller
+            hardware_source = test_context.camera_hardware_source
+            state_controller = self.__create_state_controller(test_context)
             long_exposure = 0.5
             state_controller.handle_change_profile("Snap")
             state_controller.handle_binning_changed("4")
@@ -448,8 +407,8 @@ class TestCameraControlClass(unittest.TestCase):
     def test_exception_during_view_leaves_buttons_in_ready_state(self):
         with self.__test_context() as test_context:
             document_controller = test_context.document_controller
-            hardware_source = test_context.hardware_source
-            state_controller = test_context.state_controller
+            hardware_source = test_context.camera_hardware_source
+            state_controller = self.__create_state_controller(test_context)
             play_enabled = [False]
             play_state = [None]
             def play_button_state_changed(enabled, state):
@@ -484,9 +443,9 @@ class TestCameraControlClass(unittest.TestCase):
 
     def test_profile_initialized_correctly(self):
         # this once failed due to incorrect closure handling in __update_profile_index
-        with self.__test_context(initialize=False) as test_context:
+        with self.__test_context() as test_context:
             document_controller = test_context.document_controller
-            state_controller = test_context.state_controller
+            state_controller = self.__create_state_controller(test_context, initialize=False)
             profile_ref = [None]
             def profiles_changed(profiles):
                 pass
@@ -502,8 +461,8 @@ class TestCameraControlClass(unittest.TestCase):
         with self.__test_context(is_eels=True) as test_context:
             document_controller = test_context.document_controller
             document_model = test_context.document_model
-            hardware_source = test_context.hardware_source
-            state_controller = test_context.state_controller
+            hardware_source = test_context.camera_hardware_source
+            state_controller = self.__create_state_controller(test_context)
             data_items = [None, None]
             def display_data_item_changed():
                 data_items[0] = state_controller.data_item_reference.data_item
@@ -528,8 +487,8 @@ class TestCameraControlClass(unittest.TestCase):
         with self.__test_context(is_eels=True) as test_context:
             document_controller = test_context.document_controller
             document_model = test_context.document_model
-            hardware_source = test_context.hardware_source
-            state_controller = test_context.state_controller
+            hardware_source = test_context.camera_hardware_source
+            state_controller = self.__create_state_controller(test_context)
             data_items = [None, None]
             def display_data_item_changed():
                 data_items[0] = state_controller.data_item_reference.data_item
@@ -571,7 +530,7 @@ class TestCameraControlClass(unittest.TestCase):
         with self.__test_context(is_eels=True) as test_context:
             document_controller = test_context.document_controller
             document_model = test_context.document_model
-            hardware_source = test_context.hardware_source
+            hardware_source = test_context.camera_hardware_source
             self._acquire_one(document_controller, hardware_source)
             document_model.recompute_all()
             self.assertEqual(len(document_model.data_items), 2)
@@ -588,7 +547,7 @@ class TestCameraControlClass(unittest.TestCase):
         with self.__test_context(is_eels=True) as test_context:
             document_controller = test_context.document_controller
             document_model = test_context.document_model
-            hardware_source = test_context.hardware_source
+            hardware_source = test_context.camera_hardware_source
             hardware_source.start_playing(sync_timeout=TIMEOUT)
             try:
                 start_time = time.time()
@@ -610,7 +569,7 @@ class TestCameraControlClass(unittest.TestCase):
         with self.__test_context(is_eels=True) as test_context:
             document_controller = test_context.document_controller
             document_model = test_context.document_model
-            hardware_source = test_context.hardware_source
+            hardware_source = test_context.camera_hardware_source
             self._acquire_one(document_controller, hardware_source)
             display_item = document_model.get_display_item_for_data_item(document_model.data_items[0])
             self.assertEqual(len(display_item.graphics), 1)
@@ -620,7 +579,7 @@ class TestCameraControlClass(unittest.TestCase):
         with self.__test_context(is_eels=True) as test_context:
             document_controller = test_context.document_controller
             document_model = test_context.document_model
-            hardware_source = test_context.hardware_source
+            hardware_source = test_context.camera_hardware_source
             self._acquire_one(document_controller, hardware_source)
             new_bounds = (0.45, 0.2), (0.1, 0.6)
             hardware_source.data_channels[1].processor.bounds = new_bounds
@@ -631,7 +590,7 @@ class TestCameraControlClass(unittest.TestCase):
         with self.__test_context(is_eels=True) as test_context:
             document_controller = test_context.document_controller
             document_model = test_context.document_model
-            hardware_source = test_context.hardware_source
+            hardware_source = test_context.camera_hardware_source
             self._acquire_one(document_controller, hardware_source)
             new_bounds = (0.45, 0.2), (0.1, 0.6)
             display_item = document_model.get_display_item_for_data_item(document_model.data_items[0])
@@ -642,7 +601,7 @@ class TestCameraControlClass(unittest.TestCase):
         with self.__test_context(is_eels=True) as test_context:
             document_controller = test_context.document_controller
             document_model = test_context.document_model
-            hardware_source = test_context.hardware_source
+            hardware_source = test_context.camera_hardware_source
             self._acquire_one(document_controller, hardware_source)
             display_item = document_model.get_display_item_for_data_item(document_model.data_items[0])
             display_item.remove_graphic(display_item.graphics[0])
@@ -654,7 +613,7 @@ class TestCameraControlClass(unittest.TestCase):
         with self.__test_context(is_eels=True) as test_context:
             document_controller = test_context.document_controller
             document_model = test_context.document_model
-            hardware_source = test_context.hardware_source
+            hardware_source = test_context.camera_hardware_source
             self._acquire_one(document_controller, hardware_source)
             self.assertEqual("eels", Metadata.get_metadata_value(document_model.data_items[0], "stem.signal_type"))
             self.assertEqual("eels", Metadata.get_metadata_value(document_model.data_items[1], "stem.signal_type"))
@@ -664,7 +623,7 @@ class TestCameraControlClass(unittest.TestCase):
         with self.__test_context() as test_context:
             document_controller = test_context.document_controller
             document_model = test_context.document_model
-            hardware_source = test_context.hardware_source
+            hardware_source = test_context.camera_hardware_source
             self._acquire_one(document_controller, hardware_source)
             self.assertEqual("ronchigram", Metadata.get_metadata_value(document_model.data_items[0], "stem.signal_type"))
 
@@ -672,7 +631,7 @@ class TestCameraControlClass(unittest.TestCase):
         with self.__test_context() as test_context:
             document_controller = test_context.document_controller
             document_model = test_context.document_model
-            hardware_source = test_context.hardware_source
+            hardware_source = test_context.camera_hardware_source
             stem_controller = Registry.get_component("stem_controller")
             self._acquire_one(document_controller, hardware_source)
             frame_parameters = hardware_source.get_current_frame_parameters()
@@ -694,7 +653,7 @@ class TestCameraControlClass(unittest.TestCase):
         random.seed(999)
         self.source_image = numpy.random.randn(1024, 1024).astype(numpy.float32)
         with self.__test_context() as test_context:
-            hardware_source = test_context.hardware_source
+            hardware_source = test_context.camera_hardware_source
             hardware_source.start_playing()
             try:
                 data = hardware_source.get_next_xdatas_to_finish()[0].data
@@ -711,7 +670,7 @@ class TestCameraControlClass(unittest.TestCase):
 
     def test_integrating_frames_updates_frame_count_by_integration_count(self):
         with self.__test_context() as test_context:
-            hardware_source = test_context.hardware_source
+            hardware_source = test_context.camera_hardware_source
             frame_parameters = hardware_source.get_frame_parameters(0)
             frame_parameters.integration_count = 4
             hardware_source.set_current_frame_parameters(frame_parameters)
@@ -728,13 +687,13 @@ class TestCameraControlClass(unittest.TestCase):
         with self.__test_context() as test_context:
             document_controller = test_context.document_controller
             document_model = test_context.document_model
-            hardware_source = test_context.hardware_source
+            hardware_source = test_context.camera_hardware_source
             self._acquire_one(document_controller, hardware_source)
             self.assertIsNotNone(document_model.data_items[0].timezone)
 
     def test_acquire_sequence_2d_calibrations(self):
         with self.__test_context() as test_context:
-            hardware_source = test_context.hardware_source
+            hardware_source = test_context.camera_hardware_source
             frame_parameters = hardware_source.get_frame_parameters(0)
             hardware_source.set_current_frame_parameters(frame_parameters)
             hardware_source.acquire_sequence_prepare(4)
@@ -746,7 +705,7 @@ class TestCameraControlClass(unittest.TestCase):
 
     def test_acquire_sequence_1d_calibrations(self):
         with self.__test_context() as test_context:
-            hardware_source = test_context.hardware_source
+            hardware_source = test_context.camera_hardware_source
             frame_parameters = hardware_source.get_frame_parameters(0)
             frame_parameters.processing = "sum_project"
             hardware_source.set_current_frame_parameters(frame_parameters)
@@ -762,7 +721,7 @@ class TestCameraControlClass(unittest.TestCase):
         with self.__test_context() as test_context:
             document_controller = test_context.document_controller
             document_model = test_context.document_model
-            hardware_source = test_context.hardware_source
+            hardware_source = test_context.camera_hardware_source
             stem_controller = Registry.get_component("stem_controller")
             stem_controller.validate_probe_position()
             stem_controller._update_scan_context(Geometry.IntSize(256, 256), Geometry.FloatPoint(), 12.0, 0.0)
@@ -774,7 +733,7 @@ class TestCameraControlClass(unittest.TestCase):
         with self.__test_context(is_eels=True) as test_context:
             document_controller = test_context.document_controller
             document_model = test_context.document_model
-            hardware_source = test_context.hardware_source
+            hardware_source = test_context.camera_hardware_source
             stem_controller = Registry.get_component("stem_controller")
             stem_controller.validate_probe_position()
             stem_controller._update_scan_context(Geometry.IntSize(256, 256), Geometry.FloatPoint(), 12.0, 0.0)
@@ -783,7 +742,7 @@ class TestCameraControlClass(unittest.TestCase):
 
     def test_facade_frame_parameter_methods(self):
         with self.__test_context() as test_context:
-            hardware_source = test_context.hardware_source
+            hardware_source = test_context.camera_hardware_source
             api = Facade.get_api("~1.0", "~1.0")
             hardware_source_facade = api.get_hardware_source_by_id(hardware_source.hardware_source_id, "~1.0")
             hardware_source_facade.get_default_frame_parameters()
@@ -796,7 +755,7 @@ class TestCameraControlClass(unittest.TestCase):
 
     def test_facade_playback_stop(self):
         with self.__test_context() as test_context:
-            hardware_source = test_context.hardware_source
+            hardware_source = test_context.camera_hardware_source
             api = Facade.get_api("~1.0", "~1.0")
             hardware_source_facade = api.get_hardware_source_by_id(hardware_source.hardware_source_id, "~1.0")
             self.assertFalse(hardware_source.is_playing)  # we know this works
@@ -817,7 +776,7 @@ class TestCameraControlClass(unittest.TestCase):
 
     def test_facade_playback_abort(self):
         with self.__test_context() as test_context:
-            hardware_source = test_context.hardware_source
+            hardware_source = test_context.camera_hardware_source
             api = Facade.get_api("~1.0", "~1.0")
             hardware_source_facade = api.get_hardware_source_by_id(hardware_source.hardware_source_id, "~1.0")
             self.assertFalse(hardware_source.is_playing)  # we know this works
@@ -838,7 +797,7 @@ class TestCameraControlClass(unittest.TestCase):
 
     def test_facade_record(self):
         with self.__test_context() as test_context:
-            hardware_source = test_context.hardware_source
+            hardware_source = test_context.camera_hardware_source
             api = Facade.get_api("~1.0", "~1.0")
             hardware_source_facade = api.get_hardware_source_by_id(hardware_source.hardware_source_id, "~1.0")
             self.assertFalse(hardware_source.is_recording)  # we know this works
@@ -856,7 +815,7 @@ class TestCameraControlClass(unittest.TestCase):
 
     def test_facade_abort_record(self):
         with self.__test_context() as test_context:
-            hardware_source = test_context.hardware_source
+            hardware_source = test_context.camera_hardware_source
             api = Facade.get_api("~1.0", "~1.0")
             hardware_source_facade = api.get_hardware_source_by_id(hardware_source.hardware_source_id, "~1.0")
             self.assertFalse(hardware_source.is_recording)  # we know this works
@@ -877,7 +836,7 @@ class TestCameraControlClass(unittest.TestCase):
 
     def test_facade_abort_record_and_return_data(self):
         with self.__test_context() as test_context:
-            hardware_source = test_context.hardware_source
+            hardware_source = test_context.camera_hardware_source
             api = Facade.get_api("~1.0", "~1.0")
             hardware_source_facade = api.get_hardware_source_by_id(hardware_source.hardware_source_id, "~1.0")
             data_and_metadata_list = hardware_source_facade.record()
@@ -886,7 +845,7 @@ class TestCameraControlClass(unittest.TestCase):
 
     def test_facade_view_task(self):
         with self.__test_context() as test_context:
-            hardware_source = test_context.hardware_source
+            hardware_source = test_context.camera_hardware_source
             api = Facade.get_api("~1.0", "~1.0")
             hardware_source_facade = api.get_hardware_source_by_id(hardware_source.hardware_source_id, "~1.0")
             view_task = hardware_source_facade.create_view_task()
@@ -901,7 +860,7 @@ class TestCameraControlClass(unittest.TestCase):
 
     def test_facade_record_task(self):
         with self.__test_context() as test_context:
-            hardware_source = test_context.hardware_source
+            hardware_source = test_context.camera_hardware_source
             api = Facade.get_api("~1.0", "~1.0")
             hardware_source_facade = api.get_hardware_source_by_id(hardware_source.hardware_source_id, "~1.0")
             record_task = hardware_source_facade.create_record_task()
@@ -918,7 +877,7 @@ class TestCameraControlClass(unittest.TestCase):
 
     def test_facade_record_task_cancel(self):
         with self.__test_context() as test_context:
-            hardware_source = test_context.hardware_source
+            hardware_source = test_context.camera_hardware_source
             api = Facade.get_api("~1.0", "~1.0")
             hardware_source_facade = api.get_hardware_source_by_id(hardware_source.hardware_source_id, "~1.0")
             record_task = hardware_source_facade.create_record_task()
@@ -933,7 +892,7 @@ class TestCameraControlClass(unittest.TestCase):
 
     def test_facade_grab_data(self):
         with self.__test_context() as test_context:
-            hardware_source = test_context.hardware_source
+            hardware_source = test_context.camera_hardware_source
             api = Facade.get_api("~1.0", "~1.0")
             hardware_source_facade = api.get_hardware_source_by_id(hardware_source.hardware_source_id, "~1.0")
             hardware_source_facade.start_playing()
@@ -950,6 +909,42 @@ class TestCameraControlClass(unittest.TestCase):
                 time.sleep(self.exposure * 0.1)
                 self.assertTrue(time.time() - start_time < TIMEOUT)
 
+    def test_camera_frame_parameters(self) -> None:
+        with self.__test_context() as test_context:
+            hardware_source = test_context.camera_hardware_source
+            frame_parameters = hardware_source.get_frame_parameters(0)
+            # ensure it is initially dict-like
+            self.assertEqual(frame_parameters.exposure_ms, frame_parameters["exposure_ms"])
+            self.assertEqual(frame_parameters.binning, frame_parameters["binning"])
+            self.assertEqual(frame_parameters.processing, frame_parameters["processing"])
+            self.assertEqual(frame_parameters.integration_count, frame_parameters["integration_count"])
+            # try setting values
+            frame_parameters["exposure_ms"] = 8.0
+            frame_parameters["binning"] = 8
+            frame_parameters["processing"] = "processing"
+            frame_parameters["integration_count"] = 8
+            self.assertEqual(8.0, frame_parameters.exposure_ms)
+            self.assertEqual(8, frame_parameters.binning)
+            self.assertEqual("processing", frame_parameters.processing)
+            self.assertEqual(8, frame_parameters.integration_count)
+            # ensure masks are not None and can be set to masks or dict's but always return the object
+            self.assertIsNotNone(frame_parameters.active_masks)
+            self.assertIsNotNone(frame_parameters["active_masks"])
+            mask = camera_base.Mask()
+            graphic = Graphics.RectangleGraphic()
+            mask.add_layer(graphic, 1.0)
+            frame_parameters.active_masks = [mask]
+            self.assertEqual(frame_parameters.active_masks[0].to_dict(), mask.to_dict())
+            self.assertEqual(frame_parameters["active_masks"][0].to_dict(), mask.to_dict())  # mask is already a dict
+            # test extra parameters
+            frame_parameters["extra"] = 8
+            self.assertEqual(8, frame_parameters["extra"])
+            frame_parameters_copy = camera_base.CameraFrameParameters(frame_parameters.as_dict())
+            self.assertEqual(8, frame_parameters_copy["extra"])
+            # test dict is writeable to json
+            json.dumps(frame_parameters.as_dict())
+
+
     def planned_test_custom_view_followed_by_ui_view_uses_ui_frame_parameters(self):
         pass
 
